"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const apollo_server_express_1 = require("apollo-server-express");
const loaders_1 = require("../loaders");
const logger_1 = require("../logger");
class AuthDirective extends apollo_server_express_1.SchemaDirectiveVisitor {
    visitFieldDefinition(field) {
        const { resolve } = field;
        field.resolve = async (...args) => {
            const authHandler = await new loaders_1.GqlLoader().getAuthHandler();
            const [, , context] = args;
            if (authHandler) {
                const authResult = await authHandler(context);
                if (typeof authResult === 'object') {
                    Object.assign(context, Object.assign({}, authResult));
                }
                return resolve.apply(this, args);
            }
            logger_1.logger.log('Auth decorator is used but the handler is not implemented. Please check the documentation at https://docs.code.store/getting-started/tutorials/authentication');
            return resolve.apply(this, args);
        };
    }
}
exports.default = AuthDirective;
//# sourceMappingURL=auth.directive.js.map