"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const express_1 = require("express");
const bodyParser = __importStar(require("body-parser"));
const path_1 = require("path");
const cors_1 = __importDefault(require("cors"));
const tool_1 = require("../tool");
const logger_1 = require("../logger");
class RestLoader {
    constructor(app, ctx) {
        this.app = app;
        this.ctx = ctx;
        this.handlersPath = path_1.join(tool_1.PathsResolverTool.BUILD, 'rest');
    }
    async loadEndpoints() {
        try {
            await tool_1.PromisifiedFs.access(this.handlersPath);
        }
        catch (e) {
            return [];
        }
        const files = await tool_1.PromisifiedFs.readdir(this.handlersPath);
        const restHandlers = await Promise.all(files
            .filter((file) => /.js$/.test(file))
            .map((file) => this.loadHandler(file)));
        this.applyToApplication(restHandlers);
        return restHandlers;
    }
    applyToApplication(handlers) {
        const router = express_1.Router();
        logger_1.logger.log('Loaded REST handlers:', this.constructor.name);
        handlers.forEach((handler) => {
            const { route, method, handler: fnc } = handler;
            logger_1.logger.log(`  -> ${route} - ${method.toUpperCase()}`, this.constructor.name);
            router[method](route, fnc);
        });
        this.app.use('/rest', bodyParser.json(), cors_1.default(), router);
    }
    wrap(handler) {
        return async (req, res) => {
            const { body, headers, query } = req;
            const event = {
                params: {
                    query,
                },
                body,
                headers,
            };
            try {
                const result = await handler(event, this.ctx);
                if (typeof result === 'object') {
                    if (result.status) {
                        res.status(result.status).send(result.data);
                        return;
                    }
                    res.status(200).send(result.data);
                    return;
                }
                res.status(200).send(result);
            }
            catch (e) {
                res.status(500).send(e);
            }
        };
    }
    async loadHandler(path) {
        const { default: handler } = await Promise.resolve().then(() => __importStar(require(`${this.handlersPath}/${path}`)));
        const [method, ...route] = path.replace(/\.[tj]s$/g, '').split('.');
        const wrappedHandler = this.wrap(handler);
        return {
            method,
            route: `/${route.join('/')}`,
            handler: wrappedHandler,
        };
    }
}
exports.default = RestLoader;
//# sourceMappingURL=rest.loader.js.map