"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const cli_color_1 = __importDefault(require("cli-color"));
class Logger {
    constructor() {
        var _a;
        if (((_a = process.env) === null || _a === void 0 ? void 0 : _a.LOG_OUTPUT) === 'json') {
            this.printMessage = this.printMessageAsJson;
        }
        else {
            this.printMessage = this.printMessageAsText;
        }
    }
    setServiceName(name) {
        this.serviceName = name;
    }
    error(error, trace = '', context = '', data) {
        const { stack } = new Error();
        const stackArray = stack.split('\n');
        const currentMethodIndex = stackArray.findIndex((str) => str.includes('Proxy.error'));
        const trimedStack = stackArray.slice(currentMethodIndex + 1).join('\n');
        const message = error.message || error;
        this.printMessage(message, data, cli_color_1.default.red, context, trace || trimedStack);
    }
    warn(message, context = '', data) {
        this.printMessage(message, data, cli_color_1.default.yellow, context);
    }
    log(message, context = '', data) {
        this.printMessage(message, data, cli_color_1.default.green, context);
    }
    verbose(message, context = '', data) {
        this.printMessage(message, data, cli_color_1.default.cyanBright, context);
    }
    debug(message, context = '', data) {
        this.printMessage(message, data, cli_color_1.default.magentaBright, context);
    }
    printMessageAsJson(message, data = {}, color, context) {
        const requestId = this.getFromContextStorage('requestId');
        const uniqueName = this.serviceName;
        process.stdout.write(`${JSON.stringify({
            context,
            message,
            data: Object.assign(Object.assign({}, data), { requestId,
                uniqueName }),
            timestamp: Date.now(),
        })}\n`);
    }
    setContextStorage(ctx) {
        this.contextStorage = ctx;
    }
    getFromContextStorage(field) {
        var _a;
        try {
            const store = (_a = this.contextStorage) === null || _a === void 0 ? void 0 : _a.getStore();
            return store && store[field];
        }
        catch (error) { }
    }
    printMessageAsText(message, data, color, context, trace) {
        const output = message && typeof message === 'object'
            ? `Object:\n${JSON.stringify(message, null, 2)}\n`
            : message;
        const colorOutput = color(output);
        const timestamp = new Date().toISOString();
        const contextMessage = context ? cli_color_1.default.yellow(`[${context}] `) : '';
        const dataOutput = cli_color_1.default.yellowBright(JSON.stringify(data));
        let fullMessage = `${timestamp} ${contextMessage}${/\n$/.test(output) ? colorOutput : `${colorOutput} ${dataOutput}`}\n`;
        if (trace) {
            fullMessage += `${cli_color_1.default.red(trace)}\n`;
        }
        process.stdout.write(fullMessage);
    }
}
exports.default = Logger;
//# sourceMappingURL=logger.js.map