"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.findDiff = exports.loadSchemaFields = void 0;
const graphql_file_loader_1 = require("@graphql-tools/graphql-file-loader");
const graphql_1 = require("graphql");
const load_1 = require("@graphql-tools/load");
const index_1 = require("./index");
const loaders_1 = require("../loaders");
exports.loadSchemaFields = (introspection, name) => {
    const type = introspection.__schema.types.find((it) => it.name === name);
    return type ? type.fields.map((field) => field.name) : [];
};
exports.findDiff = (loadedResolvers, schemaResolvers) => {
    if (!loadedResolvers) {
        return [[], schemaResolvers];
    }
    const resolversDiff = loadedResolvers.filter((resolver) => !schemaResolvers.find((schemaResolver) => schemaResolver === resolver));
    const schemaDiff = schemaResolvers.filter((resolver) => !loadedResolvers.find((loadedResolver) => loadedResolver === resolver));
    return [resolversDiff, schemaDiff];
};
class GraphqlValidatorTool {
    static async validateSchema() {
        const schemaPath = index_1.PathsResolverTool.SCHEMA;
        const loadSchemaOptions = { loaders: [new graphql_file_loader_1.GraphQLFileLoader()] };
        const schema = await load_1.loadSchema(schemaPath, loadSchemaOptions);
        const documentAST = graphql_1.parse(await index_1.PromisifiedFs.readFile(schemaPath, 'utf8'));
        graphql_1.validate(schema, documentAST);
    }
    static async validateQueriesAndMutations() {
        const data = await index_1.PromisifiedFs.readFile(index_1.PathsResolverTool.SCHEMA);
        const schema = graphql_1.buildSchema(data.toString());
        const introspection = graphql_1.introspectionFromSchema(schema);
        const loadedSchemaQueries = exports.loadSchemaFields(introspection, 'Query');
        const loadedSchemaMutations = exports.loadSchemaFields(introspection, 'Mutation');
        const { queryResolvers, mutationResolvers } = await new loaders_1.GqlLoader().getResolversInArray();
        const [queryResolverDiff, querySchemaDiff] = exports.findDiff(queryResolvers, loadedSchemaQueries);
        const [mutationResolverDiff, mutationSchemaDiff] = exports.findDiff(mutationResolvers, loadedSchemaMutations);
        if (queryResolverDiff.length) {
            throw new Error(`${queryResolverDiff} queries are not defined in schema`);
        }
        if (mutationResolverDiff.length) {
            throw new Error(`${mutationResolverDiff} mutations are not defined in schema`);
        }
        return {
            queries: querySchemaDiff,
            mutations: mutationSchemaDiff,
        };
    }
}
exports.default = GraphqlValidatorTool;
//# sourceMappingURL=graphql-validator.tool.js.map